require "testplan/version"
require 'rbconfig'
require 'rspec/core/rake_task'
require 'date'
require 'yaml'

spec = Gem::Specification.find_by_name 'testplan'
Dir.glob("#{spec.gem_dir}/lib/tasks/*.rake").each { |r| import r }

module Testplan
  class Setup
    def self.init(value)
      $initconf = value
    end
  end

  class Build

    def self.plan_in_format(plan, format, execute_all_at_once = false)
      self.init
      validate_plan_name(plan)
      exec_strings = []
      @@config[:testplans][plan].each do |platform|
        exec_strings += self.build_platform_exec_strings(platform, format, execute_all_at_once)
      end

      return self.execute_strings(exec_strings)

    end

    def self.platform_in_format(platform, format, execute_all_at_once = false)
      self.init
      exec_strings = self.build_platform_exec_strings(platform, format, execute_all_at_once)
      return self.execute_strings(exec_strings)
    end

    def self.init
      @@config = $initconf.call
    end

    def self.build_platform_exec_strings(platform, format, execute_all_at_once)
      envstring = make_env_string(platform)
      estrings = []

      if execute_all_at_once

        print "\nRunning all testcases at once with ENV: #{envstring}\n"

        exec_string = "#{envstring}bundle exec rake #{format.to_s} SPEC_OPTS=\""
        @@config[:platforms][platform][:cases].each do | acase |

          exec_string += " -e #{acase} "
          print "  - testcase #{acase}\n"
        end

        exec_string += "\""
        estrings << exec_string

      else
        @@config[:platforms][platform][:cases].each do | acase |

          exec_string = "#{envstring}bundle exec rake #{format.to_s} SPEC_OPTS=\"-e #{acase}\""
          print "\nRunning testcase #{acase} with ENV: #{envstring}\n"

          estrings << exec_string
        end
      end

      estrings
    end

    def self.execute_strings(exec_strings)
      exit_code = 0
      exec_strings.each do |estring|
        p estring
        exit_code += 1 unless system estring
      end

      if exit_code == 1
        raise 'At least one test failed'
      elsif exit_code > 1
        raise "There were #{exit_code} failed tests."
      end

      exit_code
    end


    def self.validate_plan_name(plan_name)
      if !@@config[:testplans].has_key?(plan_name)
         raise 'Plan name does not exist'
      end
    end

    def self.make_env_string(platform)
      envarr = {}
      @@config[:platforms][platform].each do | k,v|
        envarr[k.to_s] = v unless k == :cases
      end

      case RbConfig::CONFIG['host_os']
      when /mswin|msys|mingw|cygwin|bccwin|wince|emc/
        envstring = make_env_string_win(envarr)
      else
        envstring = make_env_string_unix(envarr)
      end

      envstring
    end

    def self.make_env_string_unix(envarr)
      envstr=''
      envarr.each do | varname,varvalue|
        envstr+= "#{varname.upcase}='#{varvalue}' "
      end
      return envstr
    end

    def self.make_env_string_win(envarr)
      envstr=''
      envarr.each do | varname,varvalue|
        envstr+= "set #{varname.upcase}=#{varvalue} &"
      end
      return envstr
    end
  end
end

if File.basename($0).to_s != 'testplan'
  if File.exists?('config/testplan.rb')
    require File.join pwd, 'config/testplan.rb'
  else
    raise "Fatal: config/testplan.rb is missing."
  end
end

